package sexpression

import error.LispException

import java.math.BigInteger

import java.text.MessageFormat.format

@DisplayName("number")
class LispNumber : Atom {

    var value: BigInteger? = null
        private set

    override val isNumber = true

    constructor(text: String) : super(text.replaceFirst("^0+(?!$)".toRegex(), "")) {
        try {
            this.value = BigInteger(text)
        } catch (e: NumberFormatException) {
            throw InvalidNumberException(text)
        }
    }

    constructor(value: BigInteger) : super(value.toString()) {
        this.value = value
    }

    class InvalidNumberException(private val text: String) : LispException() {
        override val message: String
            get() = format("{0} is not a valid integer", text)
    }

    companion object {
        val ZERO = LispNumber(BigInteger.ZERO)
        val ONE = LispNumber(BigInteger.ONE)
    }
}
