package terminal

import org.junit.jupiter.api.Assertions.assertThrows
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS
import stream.LispIOException
import java.io.IOException
import java.io.PipedInputStream
import java.io.PipedOutputStream

@TestInstance(PER_CLASS)
class TerminalConfigurationTest {

    private lateinit var configuration: TerminalConfiguration

    private fun createIOExceptionThrowingPipedOutputStream() = object : PipedOutputStream() {
        override fun connect(inputStream: PipedInputStream) = throw IOException()
    }

    @BeforeEach
    fun setUp() {
        configuration = TerminalConfiguration()
    }

    @Test
    fun setInputPairThrowsUncheckedException() {
        assertThrows(LispIOException::class.java) {
            configuration.setInputPair(createIOExceptionThrowingPipedOutputStream(), PipedInputStream())
        }
    }

    @Test
    fun setOutputPairThrowsUncheckedException() {
        assertThrows(LispIOException::class.java) {
            configuration.setOutputPair(createIOExceptionThrowingPipedOutputStream(), PipedInputStream())
        }
    }
}
